/*

    Program: smap2dp - concatenate smap images to Deskplayer's dp sequences

    GPL (C) 1999/2000  A. Schiffler

*/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
       
/* Text buffer sizes */

#define LINE_LEN	1024
#define URL_LEN		256

/* Maximum smap size (in bytes) */

#define MAX_DP_SIZE	(64*1024-1)

/* DP header magic code, default version, trailer */

#define DP_MAGIC		0x5afb
#define DP_DEFAULT_VERSION	8
#define DP_TRAILER		0xdddddddd

/* Configurable variables */

#define MAX_NUM_INPUT	256

static int numinput;
static char INPUTFILENAME[MAX_NUM_INPUT][LINE_LEN];
static char INPUTURL[MAX_NUM_INPUT][URL_LEN];
static unsigned int INPUTDELAY[MAX_NUM_INPUT];
static char OUTPUTFILENAME[LINE_LEN];

/* Setup defaults for configurable variables */

void
set_defaults ()
{
 int i;
 
 numinput=0;
 for (i=0; i<MAX_NUM_INPUT; i++) {
  strcpy (INPUTFILENAME[i], "");
  strcpy (INPUTURL[i], "");
  INPUTDELAY[i]=10;
 }
 strcpy (OUTPUTFILENAME, "");
}

void
SwapBytes (unsigned char *a, unsigned char *b)
{
  unsigned char dummy;
  dummy = *a;
  *a = *b;
  *b = dummy;
}

/* Main routine */

int
main (int argc, char *argv[])
{
  int i;
  FILE *infile;
  FILE *outfile;
  struct stat statbuffer;
  int totalsize;
  unsigned short dpmagic = DP_MAGIC;
  unsigned short dpversion = DP_DEFAULT_VERSION;
  unsigned short dptotalsize;
  unsigned char dpheader[6];
  unsigned char smapheader[4+256];
  unsigned int dptrailer = DP_TRAILER;
  int inputsize[MAX_NUM_INPUT];
  unsigned char copybuffer[MAX_DP_SIZE];
      
  /* Print message */
  printf ("smap2dp - A. Schiffler 1999\n\n");

  /* Initialize variables */
  set_defaults ();

  /* Parse command line parameters for local program */
  for (i = 1; i < argc; i++)
    {
      char *arg;
      if (argv[i][0] != '-')
	break;
      arg = argv[i];
      arg++;
      if (*arg == '-')
	arg++;
      if (strncmp (arg, "input", 5) == 0)
	{
	  i++;
	  if (i <= argc)
	    {
	      strncpy (INPUTFILENAME[numinput], argv[i], LINE_LEN);
	      numinput++;
	    }
	}
      if (strncmp (arg, "delay", 5) == 0)
	{
	  i++;
	  if (i <= argc)
	    {
	      if (numinput>0) {
 	       INPUTDELAY[numinput-1]=abs(atoi( argv[i]));
 	      }
	    }
	}
      if (strncmp (arg, "url", 3) == 0)
	{
	  i++;
	  if (i <= argc)
	    {
	      if (numinput>0) {
 	       strncpy (INPUTURL[numinput-1], argv[i], URL_LEN);
 	      }
	    }
	}
      if (strncmp (arg, "output", 6) == 0)
	{
	  i++;
	  if (i <= argc)
	    {
	      strncpy (OUTPUTFILENAME, argv[i], LINE_LEN);
	    }
	}
      if (strncmp (arg, "version", 7) == 0)
	{
	  i++;
	  if (i <= argc)
	    {
	      dpversion=abs(atoi(argv[i]));
	    }
	}
      else if (strncmp (arg, "help", 4) == 0)
	{
	 /* Command line help */
	 printf ("Usage:\n smap2dp -input name [-delay delay] [ -url url] [-input ...] [-output name] [-version #]\n");
	 printf ("\ndelay = 1/10th seconds\nurl is optional\n");
	  /* Leave */
	  exit (0);
	}
    }
  
  /* Check number of input files */
  if (numinput==0) {
   fprintf (stderr, "smap2dp: Must give at least one input file.\n");
   exit(-1);
  }  

  /* Stat all input files, scan for their size, add to total */
  totalsize=0;
  for (i=0; i<numinput; i++) {
   /* Add delaysize */
   totalsize += 4;
   /* Add URL size */
   totalsize++;
   if (strlen(INPUTURL[i])>0) {
    totalsize += 255; 
   }   
   /* Add image size */
   stat(INPUTFILENAME[i],&statbuffer);
   totalsize += statbuffer.st_size;
   /* Store size */
   inputsize[i]=statbuffer.st_size;
  }
  /* Add end-code size */
  totalsize += 4; 
    
  /* Check total dp size */
  if (totalsize>MAX_DP_SIZE) {
   fprintf (stderr, "smap2dp: Output size (%iKb) to large - must be <64Kb.\n",totalsize/1024);
   exit(-1);
  }  
  
  fprintf (stderr,"Total DP datasize is %i bytes.\n",totalsize);
  
  /* Open file for writing or set stdout */
  if (strlen (OUTPUTFILENAME) == 0)
    {
      outfile = stdout;
      strcpy (OUTPUTFILENAME, "<stdout>");
    }
  else
    {
      if ((outfile = fopen (OUTPUTFILENAME, "w")) == NULL)
	{
	  fprintf (stderr, "smap2dp: Cannot open file %s for writing.\n",
		   OUTPUTFILENAME);
	  exit (-1);
	}
    }


  /* Build dp header */
  dptotalsize=totalsize;
  memcpy(&dpheader[0],&dpmagic,2);
  memcpy(&dpheader[2],&dpversion,2);
  memcpy(&dpheader[4],&dptotalsize,2);
  SwapBytes(&dpheader[0],&dpheader[1]);
  SwapBytes(&dpheader[2],&dpheader[3]);
  SwapBytes(&dpheader[4],&dpheader[5]);
  
  /* Write header */
  fwrite (dpheader, 6, 1, outfile);
    
  /* Loop over all input images */
  for (i=0; i<numinput; i++) {
   /* Clear smapheader */
   memset(smapheader,0,sizeof(smapheader));   
   /* Build smap header */
   memcpy (&smapheader[0],&INPUTDELAY[i],4);
   smapheader[4]=(unsigned char)strlen(INPUTURL[i]);
   if (smapheader[4]>0) {   
    memcpy (&smapheader[5],INPUTURL[i],strlen(INPUTURL[i]));
    /* Write header with URL */
    fwrite (smapheader, 260, 1, outfile);
   } else {
    /* Write header without URL */
    fwrite (smapheader, 5, 1, outfile);
   }
   /* Append smap file to dp file*/
   infile=fopen(INPUTFILENAME[i],"r");
   fread(copybuffer,inputsize[i],1,infile);   
   fwrite(copybuffer,inputsize[i],1,outfile);
   fclose(infile);
  }
 
  /* Write trailer */
  fwrite (&dptrailer, 4, 1, outfile);
     
  /* Close file */
  fclose (outfile);

  exit (0);
}
